/*      $Id: //BDP/OSS/BDP_V301/BDP_Linux/oss/source/lirc/lirc-0.8.5/drivers/lirc_mtk/lirc_mtk.c#1 $      */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 10)
#error "**********************************************************"
#error " Sorry, this driver needs kernel version 2.6.10 or higher "
#error "**********************************************************"
#endif

#include <linux/version.h>
#include <linux/autoconf.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/types.h>
#include <linux/wait.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include <linux/poll.h>

#include <asm/system.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
#include <asm/uaccess.h>
#else
#include <linux/uaccess.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
#include <asm/io.h>
#else
#include <linux/io.h>
#endif

#include <linux/irq.h>
#include <linux/fcntl.h>

#include <mach/mt85xx_64b_timer.h>
#include <mach/irq_priority.h>

#include "lirc_mtk.h"
#include "drivers/lirc.h"
#include "drivers/kcompat.h"
#include "drivers/lirc_dev/lirc_dev.h"

#define LIRC_DRIVER_VERSION "$Revision: #1 $"
#define LIRC_DRIVER_NAME "lirc_mtk"

static int debug = 0;

#define dprintk(fmt, args...)					\
	do {							\
		if (debug)					\
			printk(KERN_DEBUG LIRC_DRIVER_NAME ": "	\
			       fmt, ## args);			\
	} while (0)

/* It has to be larger than 1 as well. */
#define RBUF_LEN 1024

static int reverse = 0;

static int irq = VECTOR_INT_P_GPIO1;

static HAL_TIME_T lasttv = {0, 0}; 

static struct lirc_buffer rbuf;

static void rbwrite(lirc_t l)
{
	if (lirc_buffer_full(&rbuf)) {
		/* no new signals will be accepted */
		dprintk("Buffer overrun\n");
		return;
	}
	lirc_buffer_write(&rbuf, (void *)&l);
}

static void frbwrite(lirc_t l)
{
	/* simple noise filter */
	static lirc_t pulse = 0L, space = 0L;
	static unsigned int ptr;

	if (ptr > 0 && (l & PULSE_BIT)) {
		pulse += l & PULSE_MASK;
		if (pulse > 250) {
			rbwrite(space);
			rbwrite(pulse | PULSE_BIT);
			ptr = 0;
			pulse = 0;
		}
		return;
	}
	if (!(l & PULSE_BIT)) {
		if (0 == ptr) {
			if (l > 20000) {
				space = l;
				ptr++;
				return;
			}
		} else {
			if (l > 20000) {
				space += pulse;
				if (space > PULSE_MASK)
					space = PULSE_MASK;
				space += l;
				if (space > PULSE_MASK)
					space = PULSE_MASK;
				pulse = 0;
				return;
			}
			rbwrite(space);
			rbwrite(pulse | PULSE_BIT);
			ptr = 0;
			pulse = 0;
		}
	}
	rbwrite(l);
}

static int ir_in(void)
{
     uint32_t gpioin;

	 gpioin = IR_READ32(GPIOIN);

     if(((gpioin & IR_IN) && !reverse) || (!(gpioin & IR_IN) && reverse))
	 	return 1;

	 return 0;
} 
    

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
static irqreturn_t irq_handler(int i, void *blah)
#else
static irqreturn_t irq_handler(int i, void *blah, struct pt_regs *regs)
#endif
{
	HAL_TIME_T tv;
	int dcd;
	long deltv;
	lirc_t data;
	static int last_dcd = 0;

    /* get current time */
	if(i_hal_t64b_get_time(&tv) != 0)
	{
	   printk(KERN_WARNING LIRC_DRIVER_NAME ": AIEEEE: get time fail \n");

	   goto end;
	}
		
	dcd = ir_in();
	if (dcd != last_dcd) 
	{
		/* calc time since last interrupt in microseconds */
		deltv = tv.u4Seconds-lasttv.u4Seconds;
		if (tv.u4Seconds < lasttv.u4Seconds ||
		    (tv.u4Seconds == lasttv.u4Seconds && tv.u4Micros < lasttv.u4Micros)) 
		{
			printk(KERN_WARNING LIRC_DRIVER_NAME
			       ": AIEEEE: your clock just jumped "
			       "backwards\n");
			printk(KERN_WARNING LIRC_DRIVER_NAME
			       ": %d %d %d %d %d\n",
			       dcd,
			       (int)tv.u4Seconds, (int)lasttv.u4Seconds,
			       (int)tv.u4Micros, (int)lasttv.u4Micros);
			data = PULSE_MASK;
		} 
		else if (deltv > 5) /* really long time */
		{
			data = PULSE_MASK; 
		
			dprintk(LIRC_DRIVER_NAME
			       ": AIEEEE: "
			       "%d %d %d %d %d\n",
			       dcd,
			       (int)tv.u4Seconds, (int)lasttv.u4Seconds,
			       (int)tv.u4Micros, (int)lasttv.u4Micros);
			/*
			 * detecting pulse while this
			 * MUST be a space!
			 */
		} 
		else
		{
 			data = (lirc_t) (deltv*1000000 + tv.u4Micros - lasttv.u4Micros);
		}

		frbwrite((0 == dcd)? data : (data|PULSE_BIT));
		last_dcd = dcd;
		wake_up_interruptible(&rbuf.wait_poll);
	}
    else
	{
		dprintk(LIRC_DRIVER_NAME
		": ignoring spike: %d %d %d %d %d\n",
		dcd,
		(int)tv.u4Seconds, (int)lasttv.u4Seconds,
		(int)tv.u4Micros, (int)lasttv.u4Micros);
	}
	
	lasttv = tv;

end:
	return IRQ_RETVAL(IRQ_HANDLED);
}

static int init_gpi(void)
{	
    BIM2_WRITE32(REG_RW_SINFO4_REG, (BIM2_READ32(REG_RW_SINFO4_REG) | FB_ARM1_LIRC_INIT));

    /*power down IR receiver module */
	IR_WRITE32(INTEN, (IR_READ32(INTEN) & (~(IR_INT))));
    IR_WRITE32(CONFIG_HIGH_REG, (IR_READ32(CONFIG_HIGH_REG) & (~(HWIR))));
	IR_WRITE32(CLKPDN, (IR_READ32(CLKPDN) | IRRXPD));
    
    /* Init gpi */
	if(VECTOR_INT_P_GPIO1 == irq)
	{
	    IR_WRITE32(PDMISC, ((IR_READ32(PDMISC) & (~(IR_AS_GPI_MASK)))|IR_AS_GPI_1));
		IR_WRITE32(EXINTCFG, (IR_READ32(EXINTCFG) | 0x00020000));
	}
	else if(VECTOR_INT_P_GPIO2 == irq)
	{
	    IR_WRITE32(PDMISC, ((IR_READ32(PDMISC) & (~(IR_AS_GPI_MASK)))|IR_AS_GPI_2));
		IR_WRITE32(EXINTCFG, (IR_READ32(EXINTCFG) | 0x00040000));
	}
	else if(VECTOR_INT_P_GPIO5 == irq)
	{
	    IR_WRITE32(PDMISC, ((IR_READ32(PDMISC) & (~(IR_AS_GPI_MASK)))|IR_AS_GPI_5));
		IR_WRITE32(EXINTCFG, (IR_READ32(EXINTCFG) | 0x00200000));
	}
	else if(VECTOR_INT_P_GPIO7 == irq)
	{
	    IR_WRITE32(PDMISC, ((IR_READ32(PDMISC) & (~(IR_AS_GPI_MASK)))|IR_AS_GPI_7));
		IR_WRITE32(EXINTCFG, (IR_READ32(EXINTCFG) | 0x00800000));
	}
	else
	{
	   return -EINVAL; 
	}

    BSP_SetIrqPriority(irq, IRQ_PRIORITY_MID);
	
	return 0;
}

static int set_use_inc(void *data)
{
	int result;

	/* Init read buffer. */
	if (lirc_buffer_init(&rbuf, sizeof(lirc_t), RBUF_LEN) < 0)
		return -ENOMEM;

	/* initialize timestamp */
	if(i_hal_t64b_get_time(&lasttv) != 0)
	{
		printk(KERN_WARNING LIRC_DRIVER_NAME ": AIEEEE: get time fail \n");
		return -EINVAL;
	}
	
	result = request_irq(irq, irq_handler,
			     IRQF_DISABLED,
			     LIRC_DRIVER_NAME, NULL);

	switch (result)
	{
	  case -EBUSY:
		printk(KERN_ERR LIRC_DRIVER_NAME ": IRQ %d busy\n", irq);
		lirc_buffer_free(&rbuf);
		return -EBUSY;
	  case -EINVAL:
		printk(KERN_ERR LIRC_DRIVER_NAME
		       ": Bad irq number or handler\n");
		lirc_buffer_free(&rbuf);
		return -EINVAL;
	  default:
		dprintk("Interrupt %d obtained\n", irq);
		break;
	};

	MOD_INC_USE_COUNT;
	return 0;
}

static void set_use_dec(void *data)
{
	free_irq(irq, NULL);

	dprintk("freed IRQ %d\n", irq);
	lirc_buffer_free(&rbuf);

	MOD_DEC_USE_COUNT;
}

#if 0
static void lirc_mtk_resume(void)
{
	unsigned long flags;

	local_irq_save(flags);

    if(i_hal_t64b_get_time(&lasttv) != 0)
		printk(KERN_WARNING LIRC_DRIVER_NAME ": AIEEEE: get time fail \n");
		
	lirc_buffer_clear(&rbuf);

	local_irq_restore(flags);
}
#endif

static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
		      unsigned long arg)
{
	switch (cmd) 
	{
	  default:
		return -ENOIOCTLCMD;
	}
	
	return 0;
}

static struct file_operations lirc_fops = {
	.owner	= THIS_MODULE,
	.ioctl	= lirc_ioctl,
};

static struct lirc_driver driver = {
	.name		= LIRC_DRIVER_NAME,
	.minor		= -1,
	.code_length	= 1,
	.sample_rate	= 0,
	.data		= NULL,
	.add_to_buf	= NULL,
#ifndef LIRC_REMOVE_DURING_EXPORT
	.get_queue	= NULL,
#endif
	.rbuf		= &rbuf,
	.set_use_inc	= set_use_inc,
	.set_use_dec	= set_use_dec,
	.fops		= &lirc_fops,
	.dev		= NULL,
	.owner		= THIS_MODULE,
};

#ifdef MODULE
static int __init lirc_mtk_init_module(void)
{
	int result;

    if(irq != VECTOR_INT_P_GPIO1
		&& irq != VECTOR_INT_P_GPIO2
		&& irq != VECTOR_INT_P_GPIO5
		&& irq != VECTOR_INT_P_GPIO7)
    {
      result = -EINVAL;
	  printk(KERN_ERR LIRC_DRIVER_NAME
		       ": Bad irq number or handler irq = %d\n", irq);
      goto exit_mtk_exit;
    }
 
	result = init_gpi();
	if (result < 0)
		goto exit_mtk_exit;
	driver.features = LIRC_CAN_REC_MODE2;
	driver.minor = lirc_register_driver(&driver);
	if (driver.minor < 0) {
		printk(KERN_ERR  LIRC_DRIVER_NAME
		       ": register_chrdev failed!\n");
		result = -EIO;
		goto exit_mtk_exit;
	}

	printk(KERN_INFO
	       LIRC_DRIVER_NAME " " LIRC_DRIVER_VERSION " registered\n");
	dprintk("IRQ = %d\n", irq);
	dprintk("reverse = %d\n", reverse);

	return 0;
	
exit_mtk_exit:
	return result;
}

static void __exit lirc_mtk_exit_module(void)
{
    BIM2_WRITE32(REG_RW_SINFO4_REG, (BIM2_READ32(REG_RW_SINFO4_REG) & (~FB_ARM1_LIRC_INIT)));
	
	lirc_unregister_driver(driver.minor);
	dprintk("cleaned up module\n");
}


module_init(lirc_mtk_init_module);
module_exit(lirc_mtk_exit_module);

MODULE_DESCRIPTION("Infra-red receiver driver for mtk.");
MODULE_AUTHOR("Zhifeng Zhao @ MSZ");
MODULE_LICENSE("GPL");

module_param(irq, int, S_IRUGO);
MODULE_PARM_DESC(irq, "Interrupt (97, 98, 101, or 103)");
	
module_param(reverse, bool, S_IRUGO);
MODULE_PARM_DESC(reverse, "reverse pulse and space");

module_param(debug, bool, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(debug, "Enable debugging messages");

EXPORT_NO_SYMBOLS;

#endif /* MODULE */

