
#ifndef _LIRC_MTK_H
#define _LIRC_MTK_H

#include <mach/irqs.h>
#include <mach/mt8530.h>
  
/**************************************************
    IRRX register define
    ************************************************/
#define CLKPDN              0x040
    #define IRRXPD          0x00000002  //IR receiver module clock stop

#define PDMISC              0x048
    #define IR_AS_GPI_MASK  0x000f0000  //Enable IR as GPI which can trigger interrupt
	#define IR_AS_GPI_1     0x00010000
	#define IR_AS_GPI_2     0x00020000
	#define IR_AS_GPI_5     0x00040000
	#define IR_AS_GPI_7     0x00080000

#define EXINTCFG            0x0c0
    #define EXINT_2ED_MASK	0x00ff0000  //External interrupt double edge trigger
   	#define EXINT_LEV	    0x0000ff00  //External interrupt level trigger (0: edge trigger, 1: level trigger)
                                        //This option is only valid when corresponding EXINT_2ED = 0.
	#define EXINT_POL	    0x000000ff  //External interrupt polarity 
	                                    //(0: negative edge or level low trigger, 1: positive edge or level high trigger)
	                                    //This option is only valid when corresponding EXINT_2ED = 0.

#define GPIOIN              0x0d0
    #define IR_IN           0x20000000  //IR pad data input

#define INTSTA              0x140       //INTERRUPT STATUS REGISTER
#define INTEN               0x144       //INTERRUPT ENABLE REGISTER
#define INTCLR              0x148      //INTERRUPT CLEAR REGISTER
    #define IR_INT          0x00002000
    #define GPIO_INT1       0x00000002
	#define GPIO_INT2       0x00000004
	#define GPIO_INT5       0x00000020
	#define GPIO_INT7       0x00000080	
	
#define CONFIG_HIGH_REG     0x20c
    #define HWIR            0x00000001  //Enable IR hardware receiver function.

#define IR_REGISTER_BASE         (IO_VIRT + 0x24000)

#define BIM_1_BASE                 (IO_VIRT + 0x45000)
#define REG_RW_SINFO4_REG   0x0210        //Dual Core Share Info Register 4
  #define FB_ARM1_LIRC_INIT   (1U << 3)     //[Fastboot]ARM1->ARM2 LIRC Driver Init

/**************************************************
    IO read/write macros
    ************************************************/	
/**
* Read an 8-bit register from the core
* @param _pBase core base address in memory
* @param _offset offset into the core's register space
* @return 8-bit datum
*/
#define IO_Read8(_pBase, _offset) *((volatile uint8_t *)(((uint32_t)_pBase) + _offset))

/**
* Read a 16-bit register from the core
* @param _pBase core base address in memory
* @param _offset offset into the core's register space
* @return 16-bit datum
*/
#define IO_Read16(_pBase, _offset) *((volatile uint16_t *)(((uint32_t)_pBase) + _offset))

/**
* Read a 32-bit register from the core
* @param _pBase core base address in memory
* @param _offset offset into the core's register space
* @return 32-bit datum
*/
#define IO_Read32(_pBase, _offset) *((volatile uint32_t *)(((uint32_t)_pBase) + _offset))


/**
* Write a 32-bit core register
* @param _pBase core base address in memory
* @param _offset offset into the core's register space
* @param _data 32-bit datum
*/
#define IO_Write32(_pBase, _offset, _data) \
(*((volatile uint32_t *)(((uint32_t)_pBase) + _offset)) = _data)

#define ASSERT(x)

/**************************************************
    IR device register read/write macros
    ************************************************/
#define IR_WRITE32(i4Addr, u4Val)  IO_Write32(IR_REGISTER_BASE, i4Addr, u4Val)   
#define IR_READ32(i4Addr)          IO_Read32(IR_REGISTER_BASE, i4Addr)

#define BIM2_WRITE32(i4Addr, u4Val)  IO_Write32(BIM_1_BASE, i4Addr, u4Val) 
#define BIM2_READ32(i4Addr)          IO_Read32(BIM_1_BASE, i4Addr)

#endif
